package nl.github.demo.documentation;

import static java.time.format.DateTimeFormatter.ISO_DATE_TIME;
import static java.time.temporal.ChronoUnit.SECONDS;
import static java.util.Comparator.comparing;

import io.quarkus.qute.*;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.text.Collator;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import nl.github.demo.documentation.model.Cron;
import nl.github.demo.documentation.model.PipelineSchedule;
import nl.github.demo.documentation.model.Project;
import nl.github.demo.documentation.model.ProjectSchedules;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import picocli.CommandLine.Command;

@Command(
        name = "list-schedules",
        description = "Generate an HTML file with all pipeline schedules/"
)
@Slf4j
public final class ListSchedules implements Runnable {

    @ConfigProperty(name = "output.basePath")
    Path outputPath;

    @ConfigProperty(name = "output.listSchedules")
    String htmlFileName;

    @Location("schedules.html")
    Template schedulesTemplate;

    @Override
    @SneakyThrows(IOException.class)
    public void run() {
        val projectSchedules = getProjectSchedules();
        val schedulesHtml = renderSchedulesToHtml(projectSchedules);
        writeHtmlToFile(schedulesHtml);
    }

    private ProjectSchedules getProjectSchedules() {

        val projects = provideProjects();
        projects.sort(comparing(Project::name, Collator.getInstance(Locale.ENGLISH)));
        val schedules = new HashMap<Project, List<PipelineSchedule>>();
        for (val project : projects) {
            if (!project.jobs_enabled()) {
                log.info("Ignored: Project [{}] in Group [{}], CI/CD not enabled", project.name(), project.namespace().name());
            } else {
                log.info("Added Project [{}] in Group [{}] with Id [{}]", project.name(), project.namespace().name(), project.namespace().id());
                schedules.put(project, providePipelineSchedules(project.id()));
            }
        }
        log.info("********************************************************************************");
        return new ProjectSchedules(schedules);
    }

    private String renderSchedulesToHtml(final ProjectSchedules projectSchedules) {
        return schedulesTemplate.data(
                "generationDate", createGenerationDate(),
                "schedules", projectSchedules,
                "CI_PIPELINE_ID", System.getenv("CI_PIPELINE_ID"),
                "CI_PIPELINE_URL", System.getenv("CI_PIPELINE_URL"),
                "CI_PROJECT_URL", System.getenv("CI_PROJECT_URL")
        ).render();
    }

    private RawString createGenerationDate() {
        val formattedDateTime = ISO_DATE_TIME.format(
                LocalDateTime.now().atZone(ZoneId.of("Europe/Amsterdam")).truncatedTo(SECONDS)
        );
        return new RawString(
                "<strong>%s</strong>T<strong>%s</strong>%s".formatted(
                        formattedDateTime.substring(0, 10),
                        formattedDateTime.substring(11, 20),
                        formattedDateTime.substring(20)
                )
        );
    }

    private void writeHtmlToFile(final String html) throws IOException {
        val htmlPath = outputPath.resolve(htmlFileName);
        try {
            Files.writeString(htmlPath, html);
            log.info("Schedules rendered to [{}]", htmlPath);
        } catch (final IOException ioe) {
            log.error("Could not write rendered template to [{}]" ,htmlPath, ioe);
            throw ioe;
        }
    }

    private List<Project> provideProjects() {
        List<Project> projects = new ArrayList<>();

        Project namespace = new Project(99, "namespace", null, true);

        projects.add(new Project(1, "Project 1", namespace, true));
        projects.add(new Project(2, "Project 2", namespace, true));
        projects.add(new Project(3, "Project 3", namespace, true));

        return projects;
    }

    private List<PipelineSchedule> providePipelineSchedules(int projectId) {
        List<PipelineSchedule> pipelineSchedules_1 = new ArrayList<>();
        List<PipelineSchedule> pipelineSchedules_2 = new ArrayList<>();
        List<PipelineSchedule> pipelineSchedules_3 = new ArrayList<>();

        pipelineSchedules_1.add(new PipelineSchedule("DepUpdates 1", "dev", Cron.of("0 1 * * 2"), true));
        pipelineSchedules_1.add(new PipelineSchedule("Daily build 1", "dev", Cron.of("11 1 * * 4"), true));

        pipelineSchedules_2.add(new PipelineSchedule("DepUpdates 2", "dev", Cron.of("0 2 2 * 2"), true));
        pipelineSchedules_2.add(new PipelineSchedule("Daily build 2", "dev", Cron.of("22 2 * * 5"), true));

        pipelineSchedules_3.add(new PipelineSchedule("DepUpdates 3", "dev", Cron.of("0 3 3 * 1"), true));
        pipelineSchedules_3.add(new PipelineSchedule("Daily build 3", "dev", Cron.of("33 3 * * 6"), true));

        if(projectId == 1) {
            return pipelineSchedules_1;
        } else if(projectId == 2) {
            return pipelineSchedules_2;
        } else if(projectId == 3) {
            return pipelineSchedules_3;
        }
        return Collections.emptyList();
    }

}
