package nl.github.demo.documentation.model;

import static java.lang.Integer.parseInt;
import static java.util.OptionalInt.empty;
import static java.util.stream.Collectors.joining;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import io.quarkus.qute.RawString;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.OptionalInt;

import lombok.val;
import nl.github.demo.documentation.deserializer.CronDeserializer;

@JsonDeserialize(using = CronDeserializer.class)
public record Cron(
        List<Part> minutes,
        List<Part> hours,
        List<Part> dayOfMonths,
        List<Part> months,
        List<Part> weekDays
) {

    public Cron {
        minutes = List.copyOf(minutes);
        hours = List.copyOf(hours);
        dayOfMonths = List.copyOf(dayOfMonths);
        months = List.copyOf(months);
        weekDays = List.copyOf(weekDays);
    }

    public static Cron of(final String line) {
        val parts = line.split("\\s+");
        if (parts.length != 5) {
            throw new IllegalArgumentException(
                    "A cron line should consist of five parts, got [%s] (%d parts)"
                            .formatted(line, parts.length)
            );
        }
        return new Cron(
                Arrays.stream(parts[0].split(",")).map(Part::of).toList(),
                Arrays.stream(parts[1].split(",")).map(Part::of).toList(),
                Arrays.stream(parts[2].split(",")).map(Part::of).toList(),
                Arrays.stream(parts[3].split(",")).map(Part::of).toList(),
                Arrays.stream(parts[4].split(",")).map(Part::of).toList()
        );
    }

    @Override
    public List<Part> dayOfMonths() {
        return List.copyOf(dayOfMonths);
    }

    @Override
    public List<Part> hours() {
        return List.copyOf(hours);
    }

    @Override
    public List<Part> minutes() {
        return List.copyOf(minutes);
    }

    @Override
    public List<Part> months() {
        return List.copyOf(months);
    }

    @Override
    public List<Part> weekDays() {
        return List.copyOf(weekDays);
    }

    public RawString timesOfDay() {
        val times = new ArrayList<String>();
        for (val hour : hours) {
            for (val minute : minutes) {
                times.add("%02d:%02d".formatted(hour.value, minute.value));
            }
        }
        return new RawString(String.join("<br>", times));
    }

    @Override
    public String toString() {
        return "%s %s %s %s %s".formatted(
                minutes.stream().map(Part::toString).collect(joining(",")),
                hours.stream().map(Part::toString).collect(joining(",")),
                dayOfMonths.stream().map(Part::toString).collect(joining(",")),
                months.stream().map(Part::toString).collect(joining(",")),
                weekDays.stream().map(Part::toString).collect(joining(","))
        );
    }

    public record Part(int value, OptionalInt modulo, OptionalInt offset) {
        private static final int ANY = -1;

        private static Part of(final String part) {
            val modIndex = part.indexOf('%');
            val hasModPart = (modIndex >= 0);
            val offsetIndex = part.indexOf('+');
            val hasOffsetPart = (offsetIndex >= 0);
            val isOffsetAfterMod = (offsetIndex > modIndex);
            if (hasOffsetPart && (!hasModPart || !isOffsetAfterMod)) {
                throw new IllegalArgumentException("Invalid cron line part syntax [%s]".formatted(part));
            }

            if (!hasModPart) {
                return new Part(part.equals("*") ? ANY : parseInt(part), empty(), empty());
            }

            val valuePart = part.substring(0, modIndex);
            val value = valuePart.equals("*") ? ANY : parseInt(valuePart);
            val moduloPart = part.substring(modIndex + 1, hasOffsetPart ? offsetIndex : part.length());
            val modulo = OptionalInt.of(parseInt(moduloPart));
            if (!hasOffsetPart) {
                return new Part(value, modulo, empty());
            }

            val offsetPart = part.substring(offsetIndex + 1);
            val offset = OptionalInt.of(parseInt(offsetPart));
            return new Part(value, modulo, offset);
        }

        public boolean matches(final SprintDay day) {
            if (modulo.isEmpty()) {
                return value == ANY || value == day.index() % 7;
            }
            if (modulo.getAsInt() != 2) {
                return false;
            }
            return matchesInOddWeek(day) || matchesInEvenWeek(day);
        }

        private boolean matchesInOddWeek(final SprintDay day) {
            if (offset.orElse(0) != 1) {
                return false;
            }
            return (value == ANY && day.index() > 7) || (value != ANY && value + 7 == day.index());
        }

        private boolean matchesInEvenWeek(final SprintDay day) {
            if (offset.orElse(0) != 0) {
                return false;
            }
            return (value == ANY && day.index() <= 7) || (value != ANY && value == day.index());
        }

        @Override
        public String toString() {
            return "%s%s%s".formatted(
                    value == ANY ? "*" : String.valueOf(value),
                    modulo.isPresent() ? "%" + modulo.getAsInt() : "",
                    offset.isPresent() ? "+" + offset.getAsInt() : ""
            );
        }
    }
}
