package nl.github.demo.documentation.model;

import lombok.extern.slf4j.Slf4j;

import static java.lang.Character.toLowerCase;
import static java.util.Map.Entry.comparingByKey;
import static java.util.function.Function.identity;
import static java.util.stream.Collectors.counting;
import static java.util.stream.Collectors.groupingBy;

import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.stream.Collectors;

@Slf4j
public final class ProjectSchedules {

    private final List<Project> projects;
    private final Map<Project, List<PipelineSchedule>> schedules;
    private final List<String> scheduleNames;

    public ProjectSchedules(
            Map<Project, List<PipelineSchedule>> schedules
    ) {
        this.schedules = Map.copyOf(schedules).entrySet().stream()
                .filter(projectListEntry -> !projectListEntry.getValue().isEmpty())
                .collect(Collectors.toMap(Entry::getKey, Entry::getValue));
        this.projects = this.schedules.keySet().stream().toList();
        this.scheduleNames = schedules.values().stream()
                .flatMap(List::stream)
                .map(PipelineSchedule::name)
                .collect(groupingBy(identity(), counting()))
                .entrySet().stream()
                .sorted(Entry.<String, Long>comparingByValue().reversed()
                        .thenComparing(comparingByKey()))
                .map(Entry::getKey)
                .toList();
    }

    public List<Project> getProjects() {
        return List.copyOf(this.projects);
    }

        public PipelineSchedule forProjectAndName(final Project project, final String scheduleName, final int indexCount) {
        log.info("Name: Project.Name = [{}], ScheduleName = [{}], indexCount = [{}]", project.name(), scheduleName, indexCount);
        return schedules.get(project).stream()
                .filter(schedule -> schedule.name().equals(scheduleName))
                .findFirst()
                .orElse(null);
    }

    public List<PipelineSchedule> forProjectAndDay(final Project project, final SprintDay day) {
        log.info("Day: Project.Name = [{}], SprintDay = [{}]", project.name(), day.name());
        return schedules.get(project).stream()
                .filter(schedule -> schedule.runsOn(day))
                .sorted(Comparator.<PipelineSchedule, Integer>comparing(
                                schedule -> schedule.cron().hours().get(0).value()
                        )
                        .thenComparing(schedule -> schedule.cron().minutes().get(0).value()))
                .toList();
    }

    public List<String> names() {
        return List.copyOf(scheduleNames);
    }

    public String shorten(final String name) {
        return "schedule-" + Arrays.stream(name.split("\\s+"))
                .map(s -> String.valueOf(toLowerCase(s.charAt(0))))
                .collect(Collectors.joining());
    }

    public List<SprintDay> sprintDays() {
        return InitializedOnDemand.SPRINT_DAYS;
    }

    private static class InitializedOnDemand {
        private static final List<SprintDay> SPRINT_DAYS = List.of(
                new SprintDay(1, "Monday", false),
                new SprintDay(2, "Tuesday", false),
                new SprintDay(3, "Wednesday", false),
                new SprintDay(4, "Thursday", false),
                new SprintDay(5, "Friday", false),
                new SprintDay(6, "Saturday", true),
                new SprintDay(7, "Sunday", true),
                new SprintDay(8, "Monday", false),
                new SprintDay(9, "Tuesday", false),
                new SprintDay(10, "Wednesday", false),
                new SprintDay(11, "Thursday", false),
                new SprintDay(12, "Friday", false),
                new SprintDay(13, "Saturday", true),
                new SprintDay(14, "Sunday", true)
        );
    }

}
